/***************************************************************************

                            manage queries

    -----------------------------------------------------------------------

    begin         : Sat Jul 11 20:50:53 MET 1999

    copyright     : (C) 1999-2001 Ewald Arnold <kvoctrain@ewald-arnold.de>
                    (C) 2005-2007 Peter Hedlund <peter.hedlund@kdemail.net>
                    (C) 2007 Frederik Gladhorn <frederik.gladhorn@kdemail.net>

    -----------------------------------------------------------------------

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef kvtquery_included
#define kvtquery_included

#include <QVector>

#include <keduvocgrade.h>
#include <keduvocexpression.h>

#include "prefs.h"

class KEduVocDocument;
class KEduVocExpression;
class Prefs;
class EnumCompType;
class KRandomSequence;

// // internal types, indented are subtypes
//
// #define QM_VERB           "v"    // go
// #define   QM_VERB_IRR     "ir"
// #define   QM_VERB_REG     "re"
// #define QM_NOUN           "n"    // table, coffee
// #define   QM_NOUN_F       "f"
// #define   QM_NOUN_M       "m"
// #define   QM_NOUN_S       "s"
// #define QM_NAME           "nm"
// #define QM_ART            "ar"   // article
// #define   QM_ART_DEF      "def"  // definite    a/an
// #define   QM_ART_IND      "ind"  // indefinite  the
// #define QM_ADJ            "aj"   // adjective   expensive, good
// #define QM_ADV            "av"   // adverb      today, strongly
// #define QM_PRON           "pr"   // pronoun     you, she
// #define   QM_PRON_POS     "pos"  // possessive  my, your
// #define   QM_PRON_PER     "per"  // personal
// #define QM_PHRASE         "ph"
// #define QM_NUM            "num"  // numeral
// #define   QM_NUM_ORD      "ord"  // ordinal     first, second
// #define   QM_NUM_CARD     "crd"  // cardinal    one, two
// #define QM_INFORMAL       "ifm"
// #define QM_FIG            "fig"
// #define QM_CON            "con"  // conjuncton  and, but
// #define QM_PREP           "pre"  // preposition behind, between
// #define QM_QUEST          "qu"   // question    who, what
//
// // type delimiters
//
// #define QM_USER_TYPE  "#"   // designates number of user type
// #define QM_TYPE_DIV   ":"   // divide main from subtype

// class TypeRelation
// {
// public:
//     TypeRelation(const QString & _short, const QString & _long) : shortId(_short), longId(_long)
//     {}
//
//     QString shortStr() const
//     {
//         return shortId;
//     }
//     QString longStr()  const
//     {
//         return longId;
//     }
//
// protected:
//     QString  shortId;
//     QString longId;
// };


class TestEntry
{
public:
    TestEntry(KEduVocExpression *_exp, int _nr) : exp(_exp), m_index(_nr)
    {
        m_answeredCorrectInSequence = 0;
        m_statisticCount = 0;
        m_statisticGoodCount = 0;
        m_statisticBadCount = 0;
        m_statisticTimeout = 0;
        m_statisticSkipUnknown = 0;
        m_statisticSkipKnown = 0;
        m_canSwitchDirection = false;
    }

    void incGoodCount();
    void incBadCount();
    void incTimeout();
    void incSkipUnknown();
    void incSkipKnown();

    int answeredCorrectInSequence();
    int statisticCount();
    int statisticGoodCount();
    int statisticBadCount();
    int statisticSkipKnown();
    int statisticSkipUnknown();
    int statisticTimeout();

    int totalEntries();

    ///@todo make these two members private
    KEduVocExpression *exp;
    int m_index;
private:
    void update();

    // these are for the CURRENT query only, so we can display statistics.
    int m_statisticCount;
    int m_statisticGoodCount;
    int m_statisticBadCount;
    int m_statisticTimeout;
    int m_statisticSkipUnknown;
    int m_statisticSkipKnown;
    int m_answeredCorrectInSequence;
    bool m_canSwitchDirection;
};


class TestEntryManager
{
public:
    /**
     * Create a collection of entries to be practiced.
     * @param doc
     */
    TestEntryManager(KEduVocDocument *doc);
    /**
     * Default ctor
     */
    ~TestEntryManager();

    /**
     * This is called to get the next entry to show to the user.
     * @return
     */
    TestEntry* nextEntry();

    /**
     * The user has answered with this result, internal update.
     * @param result the result (PracticeDialog)
     */
    void result(int result);

    int totalEntryCount();
    int activeEntryCount();


    /// grade int to string (?)
    static QString gradeStr(int i);

    void printStatistics();

private:

    /**
     * Check if @p expr is valid for the current query settings.
     * @param expr
     * @return
     */
    bool validate(KEduVocExpression *expr);
    bool validateWithSettings(KEduVocExpression *expr);
    bool compareBlocking(int grade, const QDateTime &limit, bool use_it);
    bool compareExpiring(int grade, const QDateTime &limit, bool use_it);
    bool compareDate(int type, const QDateTime &qd);
    bool compareQuery(int type, int qgrade, int limit);
    bool compareBad(int type, int bcount, int limit);
    bool compareGrade(int type, grade_t qgrade, grade_t limit);

    /**
     * Test if an entry has the right word type to be included in the query.
     * @param entry
     * @return
     */
    bool checkType(KEduVocExpression* entry);

    KEduVocDocument *m_doc;
    int m_fromTranslation;
    int m_toTranslation;
    int m_testType;

    /// All entries in the test.
    QList<TestEntry*> m_allTestEntries;
    /// All entries that have not been asked.
    QList<TestEntry*> m_notAskedTestEntries;
    /// The list of entries that are being asked. If one of these is done, it can be deleted and an new one from m_notAskedTestEntries taken.
    QList<TestEntry*> m_currentEntries;
    int m_currentEntry;

    KRandomSequence* m_randomSequence;
};

#endif // kvtquery_included
